#!/bin/bash
# Copyright (c) 1999-2017, Parallels International GmbH
# Copyright (c) 2018, vt@altlinux.org
#
# This file is part of OpenVZ libraries. OpenVZ is free software; you can
# redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Our contact details: Parallels International GmbH, Vordergasse 59, 8200
# Schaffhausen, Switzerland.
#
# This script configure IP alias(es) inside RedHat like VPS.
#
# Parameters are passed in environment variables.
# Required parameters:
#   IP_ADDR       - IP address(es) to add
#                   (several addresses should be divided by space)
# Optional parameters:
#   VE_STATE      - state of VPS; could be one of:
#                     starting | stopping | running | stopped
#   IPDELALL	  - delete all old interfaces
#
VENET_DEV=venet0
IFCFG=/etc/net/ifaces/$VENET_DEV
NETFILE=/etc/sysconfig/network
HOSTFILE=/etc/hosts
IFCFG_NETWORKD=/etc/systemd/network

function setup_network()
{
	rm -rf $IFCFG >/dev/null 2>&1
	put_param $NETFILE NETWORKING yes
	if [ ! -f $HOSTFILE ]; then
		echo "127.0.0.1 localhost.localdomain localhost" > $HOSTFILE
	fi
}

function setup_network_networkd()
{
	if [ ! -f $HOSTFILE ]; then
		echo "127.0.0.1 localhost.localdomain localhost" > $HOSTFILE
	fi
        echo "[Match]
Name=${VENET_DEV}
[Network]
IPv6AcceptRA=no
LinkLocalAddressing=no
DefaultRouteOnDevice=yes
DNS=8.8.8.8
DNS=8.8.4.4
" > $IFCFG_NETWORKD/${VENET_DEV}.network || \
                error "Can't write to file $IFCFG_NETWORKD/${VENET_DEV}.network" $VZ_FS_NO_DISK_SPACE
}

function init_config()
{
	mkdir -p $IFCFG
	echo "TYPE=eth
DEVICE=$VENET_DEV
BOOTPROTO=static
ONBOOT=yes" > $IFCFG/options || \
		error "Can't write to file $IFCFG/options" $VZ_FS_NO_DISK_SPACE
}

function add_ip6()
{
	local ip=$1
	local mask=$2

	put_param $IFCFG/options CONFIG_IPV6 yes
	echo "$ip/${mask:-128}" >> $IFCFG/ipv6address || \
		error "Can't write to file $IFCFG/ipv6address" ${VZ_FS_NO_DISK_SPACE}
}

function add_ip6_networkd()
{
    local ip=$1
    local mask=$2
    local ipe
    local cfg=$IFCFG_NETWORKD/${VENET_DEV}.network

    ipe=$(echo $ip | sed -e 's/[]\/$*.^[]/\\&/g')
    if grep -E "^Address=$ip/${mask:-128}" $cfg >/dev/null 2>&1; then
        echo "$ip/${mask:-128} already exists"
    else
        echo "Address=$ip/${mask:-128}" >> $cfg || \
            error "Can't write to file $cfg" ${VZ_FS_NO_DISK_SPACE}
    fi
}

function add_ip4()
{
	local ip=$1
	local mask=$2

	echo "$ip/${mask:-32}" >> $IFCFG/ipv4address || \
		error "Can't write to file $IFCFG/ipv4address" ${VZ_FS_NO_DISK_SPACE}
	echo "default dev $VENET_DEV" >"$IFCFG/ipv4route" || \
		error "Can't write to file $IFCFG/ipv4route" $VZ_FS_NO_DISK_SPACE
	echo net.ipv4.conf.$VENET_DEV.promote_secondaries=1 > ${IFCFG}/sysctl.conf || \
		error "Can't write to file $IFCFG/sysctl.conf" $VZ_FS_NO_DISK_SPACE
}

function add_ip4_networkd()
{
    local ip=$1
    local mask=$2
    local ipe
    local cfg=$IFCFG_NETWORKD/${VENET_DEV}.network

    ipe=$(echo $ip | sed -e 's/[]\/$*.^[]/\\&/g')
    if grep -E "^Address=$ip/${mask:-32}" $cfg >/dev/null 2>&1; then
        echo "$ip/${mask:-32} already exists"
    else
        echo "Address=$ip/${mask:-32}" >> $cfg || \
            error "Can't write to file $cfg" ${VZ_FS_NO_DISK_SPACE}
    fi
}

function add_ip()
{
	local ipm ip mask

	if [ "$VE_STATE" = "starting" ]; then
		setup_network
	fi
	if [ ! -d $IFCFG ]; then
		init_config
	fi
	for ipm in $IP_ADDR; do
		ip=${ipm%%/*}
		mask=
		if [[ "$ipm" =~ / ]]; then
			mask=${ipm##*/}
		fi
		if is_ipv6 $ip; then
			add_ip6 "$ip" "$mask"
		else
			add_ip4 "$ip" "$mask"
		fi
	done
	ifdown $VENET_DEV 2>/dev/null
	ifup $VENET_DEV 2>/dev/null
}

function add_ip_networkd()
{
	local ipm ip mask

	if [ "$VE_STATE" = "starting" ]; then
		setup_network_networkd
	fi
	for ipm in $IP_ADDR; do
		ip=${ipm%%/*}
		mask=
		if [[ "$ipm" =~ / ]]; then
			mask=${ipm##*/}
		fi
		if is_ipv6 $ip; then
			add_ip6_networkd "$ip" "$mask"
		else
			add_ip4_networkd "$ip" "$mask"
		fi
	done
	ifdown $VENET_DEV 2>/dev/null
	ifup $VENET_DEV 2>/dev/null
}

if [ -e /etc/systemd/system/dbus-org.freedesktop.network1.service -o \
     -e /etc/systemd/system/multi-user.target.wants/systemd-networkd.service -o \
     -e /lib/systemd/system/multi-user.target.wants/systemd-networkd.service ]; then
	add_ip_networkd
else
	add_ip
fi
exit 0
