#!/bin/bash
# Copyright (C) 1999-2015 Parallels IP Holdings GmbH
# Copyright (C) 2018 vt@altlinux.org
#
# This file is part of OpenVZ libraries. OpenVZ is free software; you can
# redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Our contact details: Parallels IP Holdings GmbH, Vordergasse 59, 8200
# Schaffhausen, Switzerland.
#
# This script configure IP alias(es) inside RedHat like VE.
#
# Parameters are passed in environment variables.
# Required parameters:
#   IP_ADDR       - IP address(es) to add
#                   (several addresses should be divided by space)
# Optional parameters:
#   VE_STATE        - state of VPS; could be one of:
#                     starting | stopping | running | stopped
#   DEVICE	    - device name
#   IPADD    	    - list of ip addresses in format <ip[/mask] | dhcp>
#   IPDEL    	    - list of ip addresses in format <ip | all>
#   GW	    	    - gateway ip
#

DEVICE=$(basename $DEVICE)
IFCFG=/etc/net/ifaces/$DEVICE
NETFILE=/etc/sysconfig/network
IFCFG_NETWORKD=/etc/systemd/network

function setup_network() # OK
{
	rm -rf $IFCFG >/dev/null 2>&1
	put_param $NETFILE NETWORKING yes
	if [ ! -f ${HOSTFILE} ]; then
		echo "127.0.0.1 localhost.localdomain localhost" > $HOSTFILE
	fi
}

function setup_network_networkd()
{
        if [ ! -f ${HOSTFILE} ]; then
                echo "127.0.0.1 localhost.localdomain localhost" > $HOSTFILE
        fi
}

function init_config()
{
	mkdir -p $IFCFG
	echo "TYPE=eth
DEVICE=$DEVICE
BOOTPROTO=static
ONBOOT=yes" > $IFCFG/options || \
		error "Can't write to file $IFCFG/options" $VZ_FS_NO_DISK_SPACE
}

# Function to delete IP address for RedHat like systems
function del_ips()
{
	local ips="$1"
	local ip ipm mask
	local file

	[ -z "$ips" ] && return 0
	[ -d $IFCFG ] || return 0

	for ipm in ${ips}; do
		ip=${ipm%%/*}
		if [ "$ip" = "all" ]; then
			rm -f $IFCFG/ipv4address
			rm -f $IFCFG/ipv6address
		else
			ipe=$(echo $ip | sed -e 's/[]\/$*.^[]/\\&/g')
			if is_ipv6 "$ip"; then
				[ -e $IFCFG/ipv6address ] && sed -i "/^$ipe\//d" $IFCFG/ipv6address
			else
				[ -e $IFCFG/ipv4address ] && sed -i "/^$ipe\//d" $IFCFG/ipv4address
			fi
		fi
		if [ ! -s $IFCFG/ipv4address ]; then
			rm -f $IFCFG/ipv4address
			rm -f $IFCFG/ipv4route
		fi
		if [ ! -s $IFCFG/ipv6address ]; then
			rm -f $IFCFG/ipv6address
			rm -f $IFCFG/ipv6route
			del_param $IFCFG/options CONFIG_IPV6
		fi
	done
}

function add_ip6()
{
	local ip=$1
	local mask=$2
	local ipe

	put_param $IFCFG/options CONFIG_IPV6 yes
	ipe=$(echo $ip | sed -e 's/[]\/$*.^[]/\\&/g')
	sed -i "/^$ipe\//d" $IFCFG/ipv6address
	echo "$ip/${mask:-128}" >> $IFCFG/ipv6address || \
		error "Can't write to file $IFCFG/ipv6address" ${VZ_FS_NO_DISK_SPACE}
}

function add_ip4()
{
	local ip=$1
	local mask=$2
	local ipe

	ipe=$(echo $ip | sed -e 's/[]\/$*.^[]/\\&/g')
	[ -e $IFCFG/ipv4address ] && sed -i "/^$ipe\//d" $IFCFG/ipv4address
	echo "$ip/${mask:-32}" >> $IFCFG/ipv4address || \
		error "Can't write to file $IFCFG/ipv4address" ${VZ_FS_NO_DISK_SPACE}
	echo net.ipv4.conf.$DEVICE.promote_secondaries=1 > $IFCFG/sysctl.conf || \
		error "Can't write to file $IFCFG/sysctl.conf" $VZ_FS_NO_DISK_SPACE
}

function add_ip4_networkd()
{
        local ip=$1
        local mask=$2
        local ipe

        ipe=$(echo $ip | sed -e 's/[]\/$*.^[]/\\&/g')
        echo "Address=$ip/${mask:-32}" >> $IFCFG_NETWORKD/${DEVICE}.network || \
                error "Can't write to file $IFCFG_NETWORKD/${DEVICE}.network" ${VZ_FS_NO_DISK_SPACE}
}

function update_dev()
{
	local ips="$1"
	local ipm ip mask

	put_param $IFCFG/options BOOTPROTO "static"
	if [ "$DHCP4" = "yes" ]; then
		check_dhcp
		put_param $IFCFG/options BOOTPROTO "dhcp"
		rm -f $IFCFG/ipv4address
		rm -f $IFCFG/ipv4route
	fi
	if [ "$DHCP6" = "yes" ]; then
		check_dhcp_ipv6
		put_param $IFCFG/options BOOTPROTO "dhcp6"
		put_param $IFCFG/options CONFIG_IPV6 "yes"
		rm -f $IFCFG/ipv6address
		rm -f $IFCFG/ipv6route
	fi
	for ipm in $ips; do
		ip=${ipm%%/*}
		if [[ "$ipm" =~ / ]]; then
			mask=${ipm##*/}
		else
			mask=
		fi
		if is_ipv6 "$ip"; then
			add_ip6 "$ip" "$mask"
		else
			add_ip4 "$ip" "$mask"
		fi
	done
	if [ -n "$IPV6" ]; then
		put_param $IFCFG/options CONFIG_IPV6 yes
	fi
}

function update_dev_systemd()
{
        local ips="$1"
        local ipm ip mask

        for ipm in $ips; do
                ip=${ipm%%/*}
                if [[ "$ipm" =~ / ]]; then
                        mask=${ipm##*/}
                else
                        mask=
                fi
                add_ip4_networkd "$ip" "$mask"
        done
}

function setup_gw()
{
	if [ -n "$GWDEL" ]; then
		rm -f $IFCFG/ipv4route
	fi
	if [ -n "$GW6DEL" ]; then
		rm -f $IFCFG/ipv6route
	fi
	if [ -n "$GW" ]; then
		echo "default dev $DEVICE via $GW" > $IFCFG/ipv4route
	fi
	if [ -n "$GW6" ]; then
		echo "default dev $DEVICE via $GW6" > $IFCFG/ipv6route
	fi
}

function setup()
{
	if [ "$VE_STATE" = "starting" ]; then
		setup_network
	fi
	if [ ! -d $IFCFG ]; then
		init_config
	fi
	[ -z "$DEVICE" ] && return
	ifdown $DEVICE 2>/dev/null
	del_ips "$IPDEL"
	update_dev "$IPADD"
	setup_gw
	ifup $DEVICE 2>/dev/null
}

function setup_networkd()
{
        if [ "$VE_STATE" = "starting" ]; then
                setup_network_networkd
        fi
        [ -z "$DEVICE" ] && return
        ifdown $DEVICE 2>/dev/null

	echo "[Match]
Name=${DEVICE}
[Network]
IPv6AcceptRA=no
LinkLocalAddressing=no
DefaultRouteOnDevice=yes
DNS=8.8.8.8
DNS=8.8.4.4
" > $IFCFG_NETWORKD/${DEVICE}.network || \
                error "Can't write to file $IFCFG_NETWORKD/${DEVICE}.network" $VZ_FS_NO_DISK_SPACE

#TODO         del_ips_networkd "$IPDEL"
        update_dev_networkd "$IPADD"

	ifup $DEVICE 2>/dev/null
}

if [ -e /etc/systemd/system/dbus-org.freedesktop.network1.service -o \
     -e /etc/systemd/system/multi-user.target.wants/systemd-networkd.service -o \
     -e /lib/systemd/system/multi-user.target.wants/systemd-networkd.service ]; then
	setup_networkd
else
	setup
fi
exit 0
